@@ -1,29 +1,30 @@
 import logging
 import os
-import sys
 import uuid
 
 from django.db import models
 from django.db.models import signals
+from django.conf import settings
 from django.contrib.contenttypes.models import ContentType
-from django.contrib.auth.models import User
 from django.core.urlresolvers import reverse
-from django.core.files.base import ContentFile
 from django.contrib.contenttypes import generic
 from django.contrib.staticfiles import finders
 from django.utils.translation import ugettext_lazy as _
 
-from geonode.security.enumerations import AUTHENTICATED_USERS, ANONYMOUS_USERS
 from geonode.layers.models import Layer
-from geonode.base.models import ResourceBase, Thumbnail, Link
+from geonode.base.models import ResourceBase, resourcebase_post_save, Link
+from geonode.documents.enumerations import DOCUMENT_TYPE_MAP, DOCUMENT_MIMETYPE_MAP
 from geonode.maps.signals import map_changed_signal
 from geonode.maps.models import Map
+from geonode.security.models import remove_object_permissions
 
 IMGTYPES = ['jpg', 'jpeg', 'tif', 'tiff', 'png', 'gif']
 
 logger = logging.getLogger(__name__)
 
+
 class Document(ResourceBase):
+
     """
     A document is any kind of information that can be attached to a map such as pdf, images, videos, xls...
     """
@@ -40,32 +41,26 @@ class Document(ResourceBase):
 
     extension = models.CharField(max_length=128, blank=True, null=True)
 
-    doc_url = models.URLField(blank=True,
-                              null=True,
-                              help_text=_('The URL of the document if it is external.'),
-                              verbose_name=_('URL'))
+    doc_type = models.CharField(max_length=128, blank=True, null=True)
 
-    popular_count = models.IntegerField(default=0)
-    share_count = models.IntegerField(default=0)
+    doc_url = models.URLField(
+        blank=True,
+        null=True,
+        help_text=_('The URL of the document if it is external.'),
+        verbose_name=_('URL'))
 
-    def __unicode__(self):  
+    def __unicode__(self):
         return self.title
-        
+
     def get_absolute_url(self):
         return reverse('document_detail', args=(self.id,))
-        
-    class Meta:
-        # custom permissions,
-        # change and delete are standard in django
-        permissions = (
-            ('view_document', 'Can view'), 
-            ('change_document_permissions', "Can change permissions"),
-        )
-
-    LEVEL_READ  = 'document_readonly'
-    LEVEL_WRITE = 'document_readwrite'
-    LEVEL_ADMIN = 'document_admin'
 
+    @property
+    def name_long(self):
+        if not self.title:
+            return str(self.id)
+        else:
+            return '%s (%s)' % (self.title, self.id)
 
     def _render_thumbnail(self):
         from cStringIO import StringIO
@@ -74,8 +69,10 @@ class Document(ResourceBase):
 
         try:
             from PIL import Image, ImageOps
-        except: 
-            logger.error('%s: Pillow not installed, cannot generate thumbnails.' % e)
+        except ImportError, e:
+            logger.error(
+                '%s: Pillow not installed, cannot generate thumbnails.' %
+                e)
             return None
 
         try:
@@ -86,18 +83,24 @@ class Document(ResourceBase):
         else:
             wand_available = True
 
-        if wand_available and self.extension and self.extension.lower() == 'pdf' and self.doc_file:
-            logger.debug('Generating a thumbnail for document: {0}'.format(self.title))
-            with image.Image(filename=self.doc_file.path) as img:
-                img.sample(*size)
-                return img.make_blob('png')
-        elif self.extension and self.extension.lower() in IMGTYPES and self.doc_file:
-            
+        if wand_available and self.extension and self.extension.lower(
+        ) == 'pdf' and self.doc_file:
+            logger.debug(
+                u'Generating a thumbnail for document: {0}'.format(
+                    self.title))
+            try:
+                with image.Image(filename=self.doc_file.path) as img:
+                    img.sample(*size)
+                    return img.make_blob('png')
+            except:
+                logger.debug('Error generating the thumbnail with Wand, cascading to a default image...')
+        # if we are still here, we use a default image thumb
+        if self.extension and self.extension.lower() in IMGTYPES and self.doc_file:
             img = Image.open(self.doc_file.path)
             img = ImageOps.fit(img, size, Image.ANTIALIAS)
         else:
             filename = finders.find('documents/{0}-placeholder.png'.format(self.extension), False) or \
-                       finders.find('documents/generic-placeholder.png', False)
+                finders.find('documents/generic-placeholder.png', False)
 
             if not filename:
                 return None
@@ -112,18 +115,32 @@ class Document(ResourceBase):
     def class_name(self):
         return self.__class__.__name__
 
+    class Meta(ResourceBase.Meta):
+        pass
+
+
 def get_related_documents(resource):
     if isinstance(resource, Layer) or isinstance(resource, Map):
         ct = ContentType.objects.get_for_model(resource)
-        return Document.objects.filter(content_type=ct,object_id=resource.pk)
-    else: return None
+        return Document.objects.filter(content_type=ct, object_id=resource.pk)
+    else:
+        return None
+
 
 def pre_save_document(instance, sender, **kwargs):
-    base_name, extension = None, None
+    base_name, extension, doc_type = None, None, None
 
     if instance.doc_file:
         base_name, extension = os.path.splitext(instance.doc_file.name)
         instance.extension = extension[1:]
+        doc_type_map = DOCUMENT_TYPE_MAP
+        doc_type_map.update(getattr(settings, 'DOCUMENT_TYPE_MAP', {}))
+        if doc_type_map is None:
+            doc_type = 'other'
+        else:
+            doc_type = doc_type_map.get(instance.extension, 'other')
+        instance.doc_type = doc_type
+
     elif instance.doc_url:
         if len(instance.doc_url) > 4 and instance.doc_url[-4] == '.':
             instance.extension = instance.doc_url[-3:]
@@ -131,15 +148,16 @@ def pre_save_document(instance, sender, **kwargs):
     if not instance.uuid:
         instance.uuid = str(uuid.uuid1())
     instance.csw_type = 'document'
-    
+
     if instance.abstract == '' or instance.abstract is None:
         instance.abstract = 'No abstract provided'
 
     if instance.title == '' or instance.title is None:
-        instance.title = instance.name
+        instance.title = instance.doc_file.name
 
     if instance.resource:
-        instance.csw_wkt_geometry = instance.resource.geographic_bounding_box.split(';')[-1]
+        instance.csw_wkt_geometry = instance.resource.geographic_bounding_box.split(
+            ';')[-1]
         instance.bbox_x0 = instance.resource.bbox_x0
         instance.bbox_x1 = instance.resource.bbox_x1
         instance.bbox_y0 = instance.resource.bbox_y0
@@ -150,44 +168,57 @@ def pre_save_document(instance, sender, **kwargs):
         instance.bbox_y0 = -90
         instance.bbox_y1 = 90
 
+
+def post_save_document(instance, *args, **kwargs):
+
+    name = None
+    ext = instance.extension
+    mime_type_map = DOCUMENT_MIMETYPE_MAP
+    mime_type_map.update(getattr(settings, 'DOCUMENT_MIMETYPE_MAP', {}))
+    mime = mime_type_map.get(ext, 'text/plain')
+    url = None
+
+    if instance.doc_file:
+        name = "Hosted Document"
+        url = '%s%s' % (
+            settings.SITEURL[:-1],
+            reverse('document_download', args=(instance.id,)))
+    elif instance.doc_url:
+        name = "External Document"
+        url = instance.doc_url
+
+    if name and url:
+        Link.objects.get_or_create(
+            resource=instance.resourcebase_ptr,
+            url=url,
+            defaults=dict(
+                extension=ext,
+                name=name,
+                mime=mime,
+                url=url,
+                link_type='data',))
+
+
 def create_thumbnail(sender, instance, created, **kwargs):
-    if not created:
-        return
+    from geonode.tasks.update import create_document_thumbnail
 
-    if instance.has_thumbnail():
-        instance.thumbnail.thumb_file.delete()
-    else:
-        instance.thumbnail = Thumbnail()
-
-    image = instance._render_thumbnail()
-     
-    instance.thumbnail.thumb_file.save('doc-%s-thumb.png' % instance.id, ContentFile(image))
-    instance.thumbnail.thumb_spec = 'Rendered'
-    instance.thumbnail.save()
-    Link.objects.get_or_create(
-        resource=instance.resourcebase_ptr,
-        url=instance.thumbnail.thumb_file.url,
-        defaults=dict(
-            name=('Thumbnail'),
-            extension='png',
-            mime='image/png',
-            link_type='image',))
+    create_document_thumbnail.delay(object_id=instance.id)
 
 
 def update_documents_extent(sender, **kwargs):
     model = 'map' if isinstance(sender, Map) else 'layer'
-    ctype = ContentType.objects.get(model= model)
+    ctype = ContentType.objects.get(model=model)
     for document in Document.objects.filter(content_type=ctype, object_id=sender.id):
         document.save()
 
-def set_missing_info(sender, instance, created, **kwargs):
-    """
-    Executes mandatory post-save logic on the Document.
-    """
 
-    instance.set_missing_info()
+def pre_delete_document(instance, sender, **kwargs):
+    remove_object_permissions(instance.get_self_resource())
+
 
 signals.pre_save.connect(pre_save_document, sender=Document)
 signals.post_save.connect(create_thumbnail, sender=Document)
-signals.post_save.connect(set_missing_info, sender=Document)
+signals.post_save.connect(post_save_document, sender=Document)
+signals.post_save.connect(resourcebase_post_save, sender=Document)
+signals.pre_delete.connect(pre_delete_document, sender=Document)
 map_changed_signal.connect(update_documents_extent)
