@@ -1,55 +1,68 @@
 import json
 import logging
 import httplib2
+import os
 
-from django.contrib.auth import authenticate, get_backends as get_auth_backends
-from django.utils import simplejson
+from django.contrib.auth import authenticate
 from django.http import HttpResponse, HttpResponseRedirect
 from django.views.decorators.http import require_POST
-from django.shortcuts import get_object_or_404, render_to_response
+from django.shortcuts import render_to_response
 from django.conf import settings
 from django.contrib.auth.decorators import user_passes_test
-from django.contrib.auth.models import User
+from django.contrib.auth import get_user_model
 from django.contrib.auth.decorators import login_required
 from django.core.urlresolvers import reverse
 from django.template import RequestContext
 from django.utils.datastructures import MultiValueDictKeyError
 from django.utils.translation import ugettext as _
 
+from guardian.shortcuts import get_objects_for_user
+
+from geonode.base.models import ResourceBase
 from geonode.layers.forms import LayerStyleUploadForm
-from geonode.layers.models import Layer
+from geonode.layers.models import Layer, Style
 from geonode.layers.views import _resolve_layer, _PERMISSION_MSG_MODIFY
 from geonode.geoserver.signals import gs_catalog
+from geonode.tasks.update import geoserver_update_layers
 from geonode.utils import json_response, _get_basic_auth_info
 from geoserver.catalog import FailedRequestError, ConflictingDataError
 from lxml import etree
-from .helpers import get_stores, gs_slurp, ogc_server_settings, set_styles, style_update
+from .helpers import get_stores, ogc_server_settings, set_styles, style_update
 
 logger = logging.getLogger(__name__)
 
+
 def stores(request, store_type=None):
     stores = get_stores(store_type)
-    data = simplejson.dumps(stores)
+    data = json.dumps(stores)
     return HttpResponse(data)
 
+
 @user_passes_test(lambda u: u.is_superuser)
 def updatelayers(request):
     params = request.REQUEST
-    #Get the owner specified in the request if any, otherwise used the logged user
+    # Get the owner specified in the request if any, otherwise used the logged
+    # user
     owner = params.get('owner', None)
-    owner = User.objects.get(username=owner) if owner is not None else request.user
+    owner = get_user_model().objects.get(
+        username=owner) if owner is not None else request.user
     workspace = params.get('workspace', None)
     store = params.get('store', None)
     filter = params.get('filter', None)
+    geoserver_update_layers.delay(ignore_errors=False, owner=owner, workspace=workspace,
+                                  store=store, filter=filter)
 
-    output = gs_slurp(ignore_errors=False, owner=owner, workspace=workspace, store=store, filter=filter)
-    return HttpResponse(simplejson.dumps(output))
+    return HttpResponseRedirect(reverse('layer_browse'))
 
 
 @login_required
 @require_POST
 def layer_style(request, layername):
-    layer = _resolve_layer(request, layername, 'layers.change_layer', _PERMISSION_MSG_MODIFY)
+    layer = _resolve_layer(
+        request,
+        layername,
+        'base.change_resourcebase',
+        _PERMISSION_MSG_MODIFY)
 
     style_name = request.POST.get('defaultStyle')
 
@@ -59,51 +72,64 @@ def layer_style(request, layername):
 
     old_default = layer.default_style
     if old_default.name == style_name:
-        return HttpResponse("Default style for %s remains %s" % (layer.name, style_name), status=200)
+        return HttpResponse(
+            "Default style for %s remains %s" %
+            (layer.name, style_name), status=200)
 
     # This code assumes without checking
     # that the new default style name is included
     # in the list of possible styles.
 
-    new_style = (style for style in layer.styles if style.name == style_name).next()
+    new_style = (
+        style for style in layer.styles if style.name == style_name).next()
 
     # Does this change this in geoserver??
     layer.default_style = new_style
-    layer.styles = [s for s in layer.styles if s.name != style_name] + [old_default]
+    layer.styles = [
+        s for s in layer.styles if s.name != style_name] + [old_default]
     layer.save()
 
-    return HttpResponse("Default style for %s changed to %s" % (layer.name, style_name),status=200)
+    return HttpResponse(
+        "Default style for %s changed to %s" %
+        (layer.name, style_name), status=200)
 
 
 @login_required
-def layer_style_upload(req, layername):
-    def respond(*args,**kw):
+def layer_style_upload(request, layername):
+    def respond(*args, **kw):
         kw['content_type'] = 'text/html'
         return json_response(*args, **kw)
-    form = LayerStyleUploadForm(req.POST, req.FILES)
+    form = LayerStyleUploadForm(request.POST, request.FILES)
     if not form.is_valid():
         return respond(errors="Please provide an SLD file.")
-    
+
     data = form.cleaned_data
-    layer = _resolve_layer(req, layername, 'layers.change_layer', _PERMISSION_MSG_MODIFY)
-    
-    sld = req.FILES['sld'].read()
+    layer = _resolve_layer(
+        request,
+        layername,
+        'base.change_resourcebase',
+        _PERMISSION_MSG_MODIFY)
+
+    sld = request.FILES['sld'].read()
 
     try:
         dom = etree.XML(sld)
-    except Exception, ex:
+    except Exception:
         return respond(errors="The uploaded SLD file is not valid XML")
-    
-    el = dom.findall("{http://www.opengis.net/sld}NamedLayer/{http://www.opengis.net/sld}Name")
+
+    el = dom.findall(
+        "{http://www.opengis.net/sld}NamedLayer/{http://www.opengis.net/sld}Name")
     if len(el) == 0 and not data.get('name'):
-        return respond(errors="Please provide a name, unable to extract one from the SLD.")
+        return respond(
+            errors="Please provide a name, unable to extract one from the SLD.")
     name = data.get('name') or el[0].text
     if data['update']:
         match = None
         styles = list(layer.styles) + [layer.default_style]
         for style in styles:
             if style.sld_name == name:
-                match = style; break
+                match = style
+                break
         if match is None:
             return respond(errors="Cannot locate style : " + name)
         match.update_body(sld)
@@ -111,17 +137,29 @@ def layer_style_upload(req, layername):
         try:
             cat = gs_catalog
             cat.create_style(name, sld)
-            layer.styles = layer.styles + [type('style', (object,), {'name': name})]
+            layer.styles = layer.styles + \
+                [type('style', (object,), {'name': name})]
             cat.save(layer.publishing)
-        except ConflictingDataError, e:
+        except ConflictingDataError:
             return respond(errors="""A layer with this name exists. Select
                                      the update option if you want to update.""")
-    return respond(body={'success': True, 'style': name, 'updated': data['update']})
+    return respond(
+        body={
+            'success': True,
+            'style': name,
+            'updated': data['update']})
+
 
 @login_required
-def layer_style_manage(req, layername):
-    layer = _resolve_layer(req, layername, 'layers.change_layer', _PERMISSION_MSG_MODIFY)
-    if req.method == 'GET':
+def layer_style_manage(request, layername):
+
+    layer = _resolve_layer(
+        request,
+        layername,
+        'layers.change_layer_style',
+        _PERMISSION_MSG_MODIFY)
+
+    if request.method == 'GET':
         try:
             cat = gs_catalog
 
@@ -130,48 +168,49 @@ def layer_style_manage(req, layername):
             try:
                 set_styles(layer, cat)
             except AttributeError:
-                logger.warn('Unable to set the default style.  Ensure Geoserver is running and that this layer exists.')
+                logger.warn(
+                    'Unable to set the default style.  Ensure Geoserver is running and that this layer exists.')
 
             all_available_gs_styles = cat.get_styles()
             gs_styles = []
             for style in all_available_gs_styles:
-                gs_styles.append(style.name)
+                gs_styles.append((style.name, style.sld_title))
 
             current_layer_styles = layer.styles.all()
             layer_styles = []
             for style in current_layer_styles:
-                layer_styles.append(style.name)
+                layer_styles.append((style.name, style.sld_title))
 
             # Render the form
             return render_to_response(
                 'layers/layer_style_manage.html',
-                RequestContext(req, {
+                RequestContext(request, {
                     "layer": layer,
                     "gs_styles": gs_styles,
                     "layer_styles": layer_styles,
-                    "default_style": layer.default_style.name
-                    }
+                    "default_style": (layer.default_style.name, layer.default_style.sld_title)
+                }
                 )
             )
         except (FailedRequestError, EnvironmentError) as e:
             msg = ('Could not connect to geoserver at "%s"'
-               'to manage style information for layer "%s"' % (
-                ogc_server_settings.LOCATION, layer.name)
-            )
+                   'to manage style information for layer "%s"' % (
+                       ogc_server_settings.LOCATION, layer.name)
+                   )
             logger.warn(msg, e)
             # If geoserver is not online, return an error
             return render_to_response(
                 'layers/layer_style_manage.html',
-                RequestContext(req, {
+                RequestContext(request, {
                     "layer": layer,
                     "error": msg
-                    }
+                }
                 )
             )
-    elif req.method == 'POST':
+    elif request.method == 'POST':
         try:
-            selected_styles = req.POST.getlist('style-select')
-            default_style = req.POST['default_style']
+            selected_styles = request.POST.getlist('style-select')
+            default_style = request.POST['default_style']
             # Save to GeoServer
             cat = gs_catalog
             gs_layer = cat.get_layer(layer.name)
@@ -185,17 +224,22 @@ def layer_style_manage(req, layername):
             # Save to Django
             layer = set_styles(layer, cat)
             layer.save()
-            return HttpResponseRedirect(reverse('layer_detail', args=(layer.typename,)))
+            return HttpResponseRedirect(
+                reverse(
+                    'layer_detail',
+                    args=(
+                        layer.service_typename,
+                    )))
         except (FailedRequestError, EnvironmentError, MultiValueDictKeyError) as e:
             msg = ('Error Saving Styles for Layer "%s"' % (layer.name)
-            )
+                   )
             logger.warn(msg, e)
             return render_to_response(
                 'layers/layer_style_manage.html',
-                RequestContext(req, {
+                RequestContext(request, {
                     "layer": layer,
                     "error": msg
-                    }
+                }
                 )
             )
 
@@ -205,13 +249,58 @@ def feature_edit_check(request, layername):
     If the layer is not a raster and the user has edit permission, return a status of 200 (OK).
     Otherwise, return a status of 401 (unauthorized).
     """
-    layer = get_object_or_404(Layer, typename=layername)
+    layer = _resolve_layer(request, layername)
     datastore = ogc_server_settings.DATASTORE
-    feature_edit = getattr(settings, "GEOGIT_DATASTORE", None) or datastore
-    if request.user.has_perm('layers.change_layer', obj=layer) and layer.storeType == 'dataStore' and feature_edit:
-        return HttpResponse(json.dumps({'authorized': True}), mimetype="application/json")
+    feature_edit = getattr(settings, "GEOGIG_DATASTORE", None) or datastore
+    if request.user.has_perm(
+            'change_layer_data',
+            obj=layer) and layer.storeType == 'dataStore' and feature_edit:
+        return HttpResponse(
+            json.dumps({'authorized': True}), mimetype="application/json")
     else:
-        return HttpResponse(json.dumps({'authorized': False}), mimetype="application/json")
+        return HttpResponse(
+            json.dumps({'authorized': False}), mimetype="application/json")
+
+
+def style_change_check(request, path):
+    """
+    If the layer has not change_layer_style permission, return a status of
+    401 (unauthorized)
+    """
+    # a new style is created with a POST and then a PUT,
+    # a style is updated with a PUT
+    # a layer is updated with a style with a PUT
+    # in both case we need to check permissions here
+    # for PUT path is /gs/rest/styles/san_andres_y_providencia_water_a452004b.xml
+    # or /ge/rest/layers/geonode:san_andres_y_providencia_coastline.json
+    # for POST path is /gs/rest/styles
+    # we will suppose that a user can create a new style only if he is an
+    # authenticated (we need to discuss about it)
+    authorized = True
+    if request.method == 'POST':
+        # new style
+        if not request.user.is_authenticated:
+            authorized = False
+    if request.method == 'PUT':
+        if path == 'rest/layers':
+            # layer update, should be safe to always authorize it
+            authorized = True
+        else:
+            # style update
+            # we will iterate all layers (should be just one if not using GS)
+            # to which the posted style is associated
+            # and check if the user has change_style_layer permissions on each of them
+            style_name = os.path.splitext(request.path)[0].split('/')[-1]
+            try:
+                style = Style.objects.get(name=style_name)
+                for layer in style.layer_styles.all():
+                    if not request.user.has_perm('change_layer_style', obj=layer):
+                        authorized = False
+            except:
+                authorized = False
+                logger.warn(
+                    'There is not a style with such a name: %s.' % style_name)
+    return authorized
 
 
 def geoserver_rest_proxy(request, proxy_path, downstream_path):
@@ -227,7 +316,7 @@ def geoserver_rest_proxy(request, proxy_path, downstream_path):
         return path[len(prefix):]
 
     path = strip_prefix(request.get_full_path(), proxy_path)
-    url = "".join([ogc_server_settings.LOCATION, downstream_path, path])
+    url = str("".join([ogc_server_settings.LOCATION, downstream_path, path]))
 
     http = httplib2.Http()
     http.add_credentials(*(ogc_server_settings.credentials))
@@ -235,19 +324,24 @@ def geoserver_rest_proxy(request, proxy_path, downstream_path):
 
     if request.method in ("POST", "PUT") and "CONTENT_TYPE" in request.META:
         headers["Content-Type"] = request.META["CONTENT_TYPE"]
+        # if user is not authorized, we must stop him
+        # we need to sync django here and check if some object (styles) can
+        # be edited by the user
+        # we should remove this geonode dependency calling layers.views straight
+        # from GXP, bypassing the proxy
+        if downstream_path in ('rest/styles', 'rest/layers') and len(request.body) > 0:
+            if not style_change_check(request, downstream_path):
+                return HttpResponse(
+                    _("You don't have permissions to change style for this layer"),
+                    mimetype="text/plain",
+                    status=401)
+            if downstream_path == 'rest/styles':
+                style_update(request, url)
 
     response, content = http.request(
         url, request.method,
-        body=request.raw_post_data or None,
+        body=request.body or None,
         headers=headers)
-        
-    # we need to sync django here
-    # we should remove this geonode dependency calling layers.views straight
-    # from GXP, bypassing the proxy
-    if downstream_path == 'rest/styles' and len(request.raw_post_data) > 0:
-        # for some reason sometime gxp sends a put with empty request
-        # need to figure out with Bart
-        style_update(request, url)
 
     return HttpResponse(
         content=content,
@@ -255,7 +349,6 @@ def geoserver_rest_proxy(request, proxy_path, downstream_path):
         mimetype=response.get("content-type", "text/plain"))
 
 
-
 def layer_batch_download(request):
     """
     batch download a set of layers
@@ -276,38 +369,41 @@ def layer_batch_download(request):
 
         def layer_son(layer):
             return {
-                "name" : layer.typename,
-                "service" : layer.service_type,
-                "metadataURL" : "",
-                "serviceURL" : ""
+                "name": layer.typename,
+                "service": layer.service_type,
+                "metadataURL": "",
+                "serviceURL": ""
             }
 
         readme = """This data is provided by GeoNode.\n\nContents:"""
+
         def list_item(lyr):
             return "%s - %s.*" % (lyr.title, lyr.name)
 
         readme = "\n".join([readme] + [list_item(l) for l in layers])
 
         fake_map = {
-            "map": { "readme": readme },
-            "layers" : [layer_son(lyr) for lyr in layers]
+            "map": {"readme": readme},
+            "layers": [layer_son(lyr) for lyr in layers]
         }
 
         url = "%srest/process/batchDownload/launch/" % ogc_server_settings.LOCATION
-        resp, content = http_client.request(url,'POST',body=json.dumps(fake_map))
+        resp, content = http_client.request(
+            url, 'POST', body=json.dumps(fake_map))
         return HttpResponse(content, status=resp.status)
 
-
     if request.method == 'GET':
         # essentially, this just proxies back to geoserver
         download_id = request.GET.get('id', None)
         if download_id is None:
             return HttpResponse(status=404)
 
-        url = "%srest/process/batchDownload/status/%s" % (ogc_server_settings.LOCATION, download_id)
-        resp,content = http_client.request(url,'GET')
+        url = "%srest/process/batchDownload/status/%s" % (
+            ogc_server_settings.LOCATION, download_id)
+        resp, content = http_client.request(url, 'GET')
         return HttpResponse(content, status=resp.status)
 
+
 def resolve_user(request):
     user = None
     geoserver = False
@@ -327,7 +423,8 @@ def resolve_user(request):
                                 status=401,
                                 mimetype="text/plain")
 
-    if not any([user, geoserver, superuser]) and not request.user.is_anonymous():
+    if not any([user, geoserver, superuser]
+               ) and not request.user.is_anonymous():
         user = request.user.username
         superuser = request.user.is_superuser
 
@@ -338,9 +435,9 @@ def resolve_user(request):
     }
 
     if acl_user and acl_user.is_authenticated():
-        resp['fullname'] = acl_user.profile.name
-        resp['email'] = acl_user.profile.email
-    return HttpResponse(json.dumps(resp))
+        resp['fullname'] = acl_user.get_full_name()
+        resp['email'] = acl_user.email
+    return HttpResponse(json.dumps(resp), mimetype="application/json")
 
 
 def layer_acls(request):
@@ -360,17 +457,19 @@ def layer_acls(request):
 
             # Nope, is it the special geoserver user?
             if (acl_user is None and
-                username == ogc_server_settings.USER and
-                password == ogc_server_settings.PASSWORD):
+                    username == ogc_server_settings.USER and
+                    password == ogc_server_settings.PASSWORD):
                 # great, tell geoserver it's an admin.
                 result = {
-                   'rw': [],
-                   'ro': [],
-                   'name': username,
-                   'is_superuser':  True,
-                   'is_anonymous': False
+                    'rw': [],
+                    'ro': [],
+                    'name': username,
+                    'is_superuser': True,
+                    'is_anonymous': False
                 }
-                return HttpResponse(json.dumps(result), mimetype="application/json")
+                return HttpResponse(
+                    json.dumps(result),
+                    mimetype="application/json")
         except Exception:
             pass
 
@@ -378,31 +477,29 @@ def layer_acls(request):
             return HttpResponse(_("Bad HTTP Authorization Credentials."),
                                 status=401,
                                 mimetype="text/plain")
-    all_readable = set()
-    all_writable = set()
-    for bck in get_auth_backends():
-        if hasattr(bck, 'objects_with_perm'):
-            all_readable.update(bck.objects_with_perm(acl_user,
-                                                      'layers.view_layer',
-                                                      Layer))
-            all_writable.update(bck.objects_with_perm(acl_user,
-                                                      'layers.change_layer',
-                                                      Layer))
-    read_only = [x for x in all_readable if x not in all_writable]
-    read_write = [x for x in all_writable if x in all_readable]
 
-    read_only = [x[0] for x in Layer.objects.filter(id__in=read_only).values_list('typename').all()]
-    read_write = [x[0] for x in Layer.objects.filter(id__in=read_write).values_list('typename').all()]
+    # Include permissions on the anonymous user
+    # use of polymorphic selectors/functions to optimize performances
+    resources_readable = get_objects_for_user(acl_user, 'view_resourcebase',
+                                              ResourceBase.objects.instance_of(Layer)).values_list('id', flat=True)
+    layer_writable = get_objects_for_user(acl_user, 'change_layer_data',
+                                          Layer.objects.all())
+
+    _read = set(Layer.objects.filter(id__in=resources_readable).values_list('typename', flat=True))
+    _write = set(layer_writable.values_list('typename', flat=True))
+
+    read_only = _read ^ _write
+    read_write = _read & _write
 
     result = {
-        'rw': read_write,
-        'ro': read_only,
+        'rw': list(read_write),
+        'ro': list(read_only),
         'name': acl_user.username,
-        'is_superuser':  acl_user.is_superuser,
+        'is_superuser': acl_user.is_superuser,
         'is_anonymous': acl_user.is_anonymous(),
     }
     if acl_user.is_authenticated():
-        result['fullname'] = acl_user.profile.name
-        result['email'] = acl_user.profile.email
+        result['fullname'] = acl_user.get_full_name()
+        result['email'] = acl_user.email
 
-    return HttpResponse(json.dumps(result), mimetype="application/json")
\ No newline at end of file
+    return HttpResponse(json.dumps(result), mimetype="application/json")
