@@ -21,37 +21,34 @@
 import os
 import shutil
 import tempfile
+import zipfile
+import StringIO
 
-from django.conf import settings
 from django.test import TestCase
-from django.test.client import Client
-from django.contrib.auth.models import User, AnonymousUser
-from django.utils import simplejson as json
 from django.core.files.uploadedfile import SimpleUploadedFile
 from django.forms import ValidationError
 from django.contrib.contenttypes.models import ContentType
 from django.core.urlresolvers import reverse
 from django.db.models import Count
+from django.contrib.auth import get_user_model
 from agon_ratings.models import OverallRating
 
-import geonode.layers.utils
-import geonode.layers.views
-import geonode.layers.models
+from guardian.shortcuts import get_anonymous_user
+from guardian.shortcuts import assign_perm, remove_perm
 
 from geonode import GeoNodeException
-
 from geonode.layers.models import Layer, Style
-from geonode.layers.forms import JSONField, LayerUploadForm
 from geonode.layers.utils import layer_type, get_files, get_valid_name, \
-                                get_valid_layer_name
+    get_valid_layer_name
 from geonode.people.utils import get_valid_user
-from geonode.security.enumerations import ANONYMOUS_USERS, AUTHENTICATED_USERS
 from geonode.base.models import TopicCategory
-from geonode.search.populate_search_test_data import create_models
+from geonode.base.populate_test_data import create_models, all_public
+from geonode.layers.forms import JSONField, LayerUploadForm
 from .populate_layers_data import create_layer_data
 
 
 class LayersTest(TestCase):
+
     """Tests geonode.layers app/module
     """
 
@@ -62,218 +59,65 @@ class LayersTest(TestCase):
         self.passwd = 'admin'
         create_models(type='layer')
         create_layer_data()
-
-    # Permissions Tests
-
-    # Users
-    # - admin (pk=2)
-    # - bobby (pk=1)
-
-    # Inherited
-    # - LEVEL_NONE = _none
-
-    # Layer
-    # - LEVEL_READ = layer_read
-    # - LEVEL_WRITE = layer_readwrite
-    # - LEVEL_ADMIN = layer_admin
-
-    # Map
-    # - LEVEL_READ = map_read
-    # - LEVEL_WRITE = map_readwrite
-    # - LEVEL_ADMIN = map_admin
-
-
-    # FIXME: Add a comprehensive set of permissions specifications that allow us
-    # to test as many conditions as is possible/necessary
-
-    # If anonymous and/or authenticated are not specified,
-    # should set_layer_permissions remove any existing perms granted??
-
-    perm_spec = {"anonymous":"_none","authenticated":"_none","users":[["admin","layer_readwrite"]]}
-    def test_layer_set_default_permissions(self):
-        """Verify that Layer.set_default_permissions is behaving as expected
-        """
-
-        # Get a Layer object to work with
-        layer = Layer.objects.all()[0]
-
-        # Set the default permissions
-        layer.set_default_permissions()
-
-        # Save the layers Current Permissions
-        current_perms = layer.get_all_level_info()
-
-        # Test that LEVEL_READ is set for ANONYMOUS_USERS and AUTHENTICATED_USERS
-        self.assertEqual(layer.get_gen_level(ANONYMOUS_USERS), layer.LEVEL_READ)
-        self.assertEqual(layer.get_gen_level(AUTHENTICATED_USERS), layer.LEVEL_READ)
-
-        admin_perms = current_perms['users'][layer.owner.username]
-
-        # Test that the owner was assigned LEVEL_ADMIN
-        self.assertEqual(admin_perms, layer.LEVEL_ADMIN)
-
-    def test_set_layer_permissions(self):
-        """Verify that the set_layer_permissions view is behaving as expected
-        """
-
-        # Get a layer to work with
-        layer = Layer.objects.all()[0]
-
-        # FIXME Test a comprehensive set of permissions specifications
-
-        # Set the Permissions
-
-        layer.set_permissions(self.perm_spec)
-
-        # Test that the Permissions for ANONYMOUS_USERS and AUTHENTICATED_USERS were set correctly
-        self.assertEqual(layer.get_gen_level(ANONYMOUS_USERS), layer.LEVEL_NONE)
-        self.assertEqual(layer.get_gen_level(AUTHENTICATED_USERS), layer.LEVEL_NONE)
-
-        # Test that previous permissions for users other than ones specified in
-        # the perm_spec (and the layers owner) were removed
-        users = [n[0] for n in self.perm_spec['users']]
-        levels = layer.get_user_levels().exclude(user__username__in = users + [layer.owner])
-        self.assertEqual(len(levels), 0)
-
-        # Test that the User permissions specified in the perm_spec were applied properly
-        for username, level in self.perm_spec['users']:
-            user = geonode.maps.models.User.objects.get(username=username)
-            self.assertEqual(layer.get_user_level(user), level)
-
-    def test_ajax_layer_permissions(self):
-        """Verify that the ajax_layer_permissions view is behaving as expected
-        """
-
-        # Setup some layer names to work with
-        valid_layer_typename = Layer.objects.all()[0].id
-        invalid_layer_id = 9999999
-
-        c = Client()
-
-        # Test that an invalid layer.typename is handled for properly
-        response = c.post(reverse('resource_permissions', args=('layer', invalid_layer_id,)),
-                            data=json.dumps(self.perm_spec),
-                            content_type="application/json")
-        self.assertEquals(response.status_code, 404)
-
-        # Test that GET returns permissions
-        response = c.get(reverse('resource_permissions', args=('layer', valid_layer_typename,)))
-        assert('permissions' in response.content)
-
-        # Test that a user is required to have maps.change_layer_permissions
-
-        # First test un-authenticated
-        response = c.post(reverse('resource_permissions', args=('layer', valid_layer_typename,)),
-                            data=json.dumps(self.perm_spec),
-                            content_type="application/json")
-        self.assertEquals(response.status_code, 401)
-
-        # Next Test with a user that does NOT have the proper perms
-        logged_in = c.login(username='bobby', password='bob')
-        self.assertEquals(logged_in, True)
-        response = c.post(reverse('resource_permissions', args=('layer', valid_layer_typename,)),
-                            data=json.dumps(self.perm_spec),
-                            content_type="application/json")
-        self.assertEquals(response.status_code, 401)
-
-        # Login as a user with the proper permission and test the endpoint
-        logged_in = c.login(username='admin', password='admin')
-        self.assertEquals(logged_in, True)
-
-        response = c.post(reverse('resource_permissions', args=('layer', valid_layer_typename,)),
-                            data=json.dumps(self.perm_spec),
-                            content_type="application/json")
-
-        # Test that the method returns 200
-        self.assertEquals(response.status_code, 200)
-
-        # Test that the permissions specification is applied
-
-        # Should we do this here, or assume the tests in
-        # test_set_layer_permissions will handle for that?
-
-    def test_perms_info(self):
-        """ Verify that the perms_info view is behaving as expected
-        """
-
-        # Test with a Layer object
-        layer = Layer.objects.all()[0]
-        layer_info = layer.get_all_level_info()
-        info = geonode.security.views._perms_info(layer, geonode.layers.views.LAYER_LEV_NAMES)
-
-        # Test that ANONYMOUS_USERS and AUTHENTICATED_USERS are set properly
-        self.assertEqual(info[ANONYMOUS_USERS], layer.LEVEL_READ)
-        self.assertEqual(info[AUTHENTICATED_USERS], layer.LEVEL_READ)
-
-        self.assertEqual(info['users'], sorted(layer_info['users'].items()))
-
-        # Test that layer owner can edit layer
-        self.assertTrue(layer.owner.has_perm(set([u'layers.change_layer']), layer))
-
-        # TODO Much more to do here once jj0hns0n understands the ACL system better
-
-        # Test with a Map object
-        # TODO
+        self.anonymous_user = get_anonymous_user()
 
     # Data Tests
 
     def test_data(self):
         '''/data/ -> Test accessing the data page'''
-        c = Client()
-        response = c.get(reverse('layer_browse'))
+        response = self.client.get(reverse('layer_browse'))
         self.failUnlessEqual(response.status_code, 200)
 
     def test_describe_data_2(self):
-        '''/data/base:CA/metadata -> Test accessing the description of a layer '''
-        self.assertEqual(7, User.objects.all().count())
-        c = Client()
-        response = c.get(reverse('layer_metadata', args=('base:CA',)))
+        '''/data/geonode:CA/metadata -> Test accessing the description of a layer '''
+        self.assertEqual(8, get_user_model().objects.all().count())
+        response = self.client.get(reverse('layer_metadata', args=('geonode:CA',)))
         # Since we are not authenticated, we should not be able to access it
         self.failUnlessEqual(response.status_code, 302)
         # but if we log in ...
-        c.login(username='admin', password='admin')
+        self.client.login(username='admin', password='admin')
         # ... all should be good
-        response = c.get(reverse('layer_metadata', args=('base:CA',)))
+        response = self.client.get(reverse('layer_metadata', args=('geonode:CA',)))
         self.failUnlessEqual(response.status_code, 200)
 
     # Layer Tests
 
     # Test layer upload endpoint
     def test_upload_layer(self):
-        c = Client()
 
         # Test redirection to login form when not logged in
-        response = c.get(reverse('layer_upload'))
-        self.assertEquals(response.status_code,302)
+        response = self.client.get(reverse('layer_upload'))
+        self.assertEquals(response.status_code, 302)
         # Test return of upload form when logged in
-        c.login(username="bobby", password="bob")
-        response = c.get(reverse('layer_upload'))
-        self.assertEquals(response.status_code,200)
+        self.client.login(username="bobby", password="bob")
+        response = self.client.get(reverse('layer_upload'))
+        self.assertEquals(response.status_code, 200)
 
     def test_describe_data(self):
-        '''/data/base:CA/metadata -> Test accessing the description of a layer '''
-        self.assertEqual(7, User.objects.all().count())
-        c = Client()
-        response = c.get(reverse('layer_metadata', args=('base:CA',)))
+        '''/data/geonode:CA/metadata -> Test accessing the description of a layer '''
+        self.assertEqual(8, get_user_model().objects.all().count())
+        response = self.client.get(reverse('layer_metadata', args=('geonode:CA',)))
         # Since we are not authenticated, we should not be able to access it
         self.failUnlessEqual(response.status_code, 302)
         # but if we log in ...
-        c.login(username='admin', password='admin')
+        self.client.login(username='admin', password='admin')
         # ... all should be good
-        response = c.get(reverse('layer_metadata', args=('base:CA',)))
+        response = self.client.get(reverse('layer_metadata', args=('geonode:CA',)))
         self.failUnlessEqual(response.status_code, 200)
 
     def test_layer_attributes(self):
         lyr = Layer.objects.get(pk=1)
-        #There should be a total of 3 attributes
+        # There should be a total of 3 attributes
         self.assertEqual(len(lyr.attribute_set.all()), 4)
-        #2 out of 3 attributes should be visible
+        # 2 out of 3 attributes should be visible
         custom_attributes = lyr.attribute_set.visible()
         self.assertEqual(len(custom_attributes), 3)
-        #place_ name should come before description
+        # place_ name should come before description
         self.assertEqual(custom_attributes[0].attribute_label, "Place Name")
         self.assertEqual(custom_attributes[1].attribute_label, "Description")
-        self.assertEqual(custom_attributes[2].attribute, u'N\xfamero_De_M\xe9dicos')
+        self.assertEqual(
+            custom_attributes[2].attribute,
+            u'N\xfamero_De_M\xe9dicos')
         # TODO: do test against layer with actual attribute statistics
         self.assertEqual(custom_attributes[1].count, 1)
         self.assertEqual(custom_attributes[1].min, "NA")
@@ -287,45 +131,57 @@ class LayersTest(TestCase):
     def test_layer_attribute_config(self):
         lyr = Layer.objects.get(pk=1)
         custom_attributes = (lyr.attribute_config())["getFeatureInfo"]
-        self.assertEqual(custom_attributes["fields"],["place_name","description", u'N\xfamero_De_M\xe9dicos'])
-        self.assertEqual(custom_attributes["propertyNames"]["description"], "Description")
-        self.assertEqual(custom_attributes["propertyNames"]["place_name"], "Place Name")
+        self.assertEqual(
+            custom_attributes["fields"], [
+                "place_name", "description", u'N\xfamero_De_M\xe9dicos'])
+        self.assertEqual(
+            custom_attributes["propertyNames"]["description"],
+            "Description")
+        self.assertEqual(
+            custom_attributes["propertyNames"]["place_name"],
+            "Place Name")
 
     def test_layer_styles(self):
         lyr = Layer.objects.get(pk=1)
-        #There should be a total of 3 styles
+        # There should be a total of 3 styles
         self.assertEqual(len(lyr.styles.all()), 4)
-        #One of the style is the default one
-        self.assertEqual(lyr.default_style, Style.objects.get(id=lyr.default_style.id))
+        # One of the style is the default one
+        self.assertEqual(
+            lyr.default_style,
+            Style.objects.get(
+                id=lyr.default_style.id))
 
         try:
-            styles = [str(style) for style in lyr.styles.all()]
+            [str(style) for style in lyr.styles.all()]
         except UnicodeEncodeError:
-            self.fail("str of the Style model throws a UnicodeEncodeError with special characters.")
+            self.fail(
+                "str of the Style model throws a UnicodeEncodeError with special characters.")
 
     def test_layer_save(self):
         lyr = Layer.objects.get(pk=1)
         lyr.keywords.add(*["saving", "keywords"])
         lyr.save()
-        self.assertEqual(lyr.keyword_list(), ["populartag", "here", "keywords", "saving"])
+        self.assertEqual(
+            lyr.keyword_list(), [
+                "populartag", "here", "keywords", "saving"])
 
     def test_get_valid_user(self):
         # Verify it accepts an admin user
-        adminuser = User.objects.get(is_superuser=True)
+        adminuser = get_user_model().objects.get(is_superuser=True)
         valid_user = get_valid_user(adminuser)
         msg = ('Passed in a valid admin user "%s" but got "%s" in return'
-                % (adminuser, valid_user))
+               % (adminuser, valid_user))
         assert valid_user.id == adminuser.id, msg
 
         # Verify it returns a valid user after receiving None
         valid_user = get_valid_user(None)
         msg = ('Expected valid user after passing None, got "%s"' % valid_user)
-        assert isinstance(valid_user, User), msg
+        assert isinstance(valid_user, get_user_model()), msg
 
-        newuser = User.objects.create(username='arieluser')
+        newuser = get_user_model().objects.create(username='arieluser')
         valid_user = get_valid_user(newuser)
         msg = ('Passed in a valid user "%s" but got "%s" in return'
-                % (newuser, valid_user))
+               % (newuser, valid_user))
         assert valid_user.id == newuser.id, msg
 
         valid_user = get_valid_user('arieluser')
@@ -333,7 +189,7 @@ class LayersTest(TestCase):
                ' "%s" in return' % ('arieluser', valid_user))
         assert valid_user.username == 'arieluser', msg
 
-        nn = AnonymousUser()
+        nn = get_anonymous_user()
         self.assertRaises(GeoNodeException, get_valid_user, nn)
 
     def testShapefileValidation(self):
@@ -409,6 +265,19 @@ class LayersTest(TestCase):
         files = dict(base_file=SimpleUploadedFile('foo.GEOTIF', ' '))
         self.assertTrue(LayerUploadForm(dict(), files).is_valid())
 
+    def testZipValidation(self):
+        the_zip = zipfile.ZipFile('test_upload.zip', 'w')
+        in_memory_file = StringIO.StringIO()
+        in_memory_file.write('test')
+        the_zip.writestr('foo.shp', in_memory_file.getvalue())
+        the_zip.writestr('foo.dbf', in_memory_file.getvalue())
+        the_zip.writestr('foo.shx', in_memory_file.getvalue())
+        the_zip.writestr('foo.prj', in_memory_file.getvalue())
+        the_zip.close()
+        files = dict(base_file=SimpleUploadedFile('test_upload.zip', open('test_upload.zip').read()))
+        self.assertTrue(LayerUploadForm(dict(), files).is_valid())
+        os.remove('test_upload.zip')
+
     def testWriteFiles(self):
         files = dict(
             base_file=SimpleUploadedFile('foo.shp', ' '),
@@ -420,8 +289,23 @@ class LayersTest(TestCase):
 
         tempdir = form.write_files()[0]
         self.assertEquals(set(os.listdir(tempdir)),
-            set(['foo.shp', 'foo.shx', 'foo.dbf', 'foo.prj']))
+                          set(['foo.shp', 'foo.shx', 'foo.dbf', 'foo.prj']))
 
+        the_zip = zipfile.ZipFile('test_upload.zip', 'w')
+        in_memory_file = StringIO.StringIO()
+        in_memory_file.write('test')
+        the_zip.writestr('foo.shp', in_memory_file.getvalue())
+        the_zip.writestr('foo.dbf', in_memory_file.getvalue())
+        the_zip.writestr('foo.shx', in_memory_file.getvalue())
+        the_zip.writestr('foo.prj', in_memory_file.getvalue())
+        the_zip.close()
+        files = dict(base_file=SimpleUploadedFile('test_upload.zip', open('test_upload.zip').read()))
+        form = LayerUploadForm(dict(), files)
+        self.assertTrue(form.is_valid())
+        tempdir = form.write_files()[0]
+        self.assertEquals(set(os.listdir(tempdir)),
+                          set(['foo.shp', 'foo.shx', 'foo.dbf', 'foo.prj']))
+        os.remove('test_upload.zip')
 
     def test_layer_type(self):
         self.assertEquals(layer_type('foo.shp'), 'vector')
@@ -455,14 +339,16 @@ class LayersTest(TestCase):
                 open(path, 'w').close()
 
             gotten_files = get_files(os.path.join(d, "foo.shp"))
-            gotten_files = dict((k, v[len(d) + 1:]) for k, v in gotten_files.iteritems())
-            self.assertEquals(gotten_files, dict(base="foo.shp", shp="foo.shp", shx="foo.shx",
-                prj="foo.prj", dbf="foo.dbf"))
+            gotten_files = dict((k, v[len(d) + 1:])
+                                for k, v in gotten_files.iteritems())
+            self.assertEquals(gotten_files, dict(shp="foo.shp", shx="foo.shx",
+                                                 prj="foo.prj", dbf="foo.dbf"))
         finally:
             if d is not None:
                 shutil.rmtree(d)
 
-        # Check that a Shapefile missing required components raises an exception
+        # Check that a Shapefile missing required components raises an
+        # exception
         d = None
         try:
             d = tempfile.mkdtemp()
@@ -471,12 +357,18 @@ class LayersTest(TestCase):
                 # open and immediately close to create empty file
                 open(path, 'w').close()
 
-            self.assertRaises(GeoNodeException, lambda: get_files(os.path.join(d, "foo.shp")))
+            self.assertRaises(
+                GeoNodeException,
+                lambda: get_files(
+                    os.path.join(
+                        d,
+                        "foo.shp")))
         finally:
             if d is not None:
                 shutil.rmtree(d)
 
-        # Check that including an SLD with a valid shapefile results in the SLD getting picked up
+        # Check that including an SLD with a valid shapefile results in the SLD
+        # getting picked up
         d = None
         try:
             d = tempfile.mkdtemp()
@@ -486,9 +378,16 @@ class LayersTest(TestCase):
                 open(path, 'w').close()
 
             gotten_files = get_files(os.path.join(d, "foo.shp"))
-            gotten_files = dict((k, v[len(d) + 1:]) for k, v in gotten_files.iteritems())
-            self.assertEquals(gotten_files, dict(base="foo.shp", shp="foo.shp", shx="foo.shx",
-                prj="foo.prj", dbf="foo.dbf", sld="foo.sld"))
+            gotten_files = dict((k, v[len(d) + 1:])
+                                for k, v in gotten_files.iteritems())
+            self.assertEquals(
+                gotten_files,
+                dict(
+                    shp="foo.shp",
+                    shx="foo.shx",
+                    prj="foo.prj",
+                    dbf="foo.dbf",
+                    sld="foo.sld"))
         finally:
             if d is not None:
                 shutil.rmtree(d)
@@ -503,9 +402,10 @@ class LayersTest(TestCase):
                 open(path, 'w').close()
 
             gotten_files = get_files(os.path.join(d, "foo.SHP"))
-            gotten_files = dict((k, v[len(d) + 1:]) for k, v in gotten_files.iteritems())
-            self.assertEquals(gotten_files, dict(base="foo.SHP", shp="foo.SHP", shx="foo.SHX",
-                prj="foo.PRJ", dbf="foo.DBF"))
+            gotten_files = dict((k, v[len(d) + 1:])
+                                for k, v in gotten_files.iteritems())
+            self.assertEquals(gotten_files, dict(shp="foo.SHP", shx="foo.SHX",
+                                                 prj="foo.PRJ", dbf="foo.DBF"))
         finally:
             if d is not None:
                 shutil.rmtree(d)
@@ -520,18 +420,28 @@ class LayersTest(TestCase):
                 open(path, 'w').close()
 
             gotten_files = get_files(os.path.join(d, "foo.SHP"))
-            gotten_files = dict((k, v[len(d) + 1:]) for k, v in gotten_files.iteritems())
-            self.assertEquals(gotten_files, dict(base="foo.SHP", shp="foo.SHP", shx="foo.shx",
-                prj="foo.pRJ", dbf="foo.DBF"))
+            gotten_files = dict((k, v[len(d) + 1:])
+                                for k, v in gotten_files.iteritems())
+            self.assertEquals(gotten_files, dict(shp="foo.SHP", shx="foo.shx",
+                                                 prj="foo.pRJ", dbf="foo.DBF"))
         finally:
             if d is not None:
                 shutil.rmtree(d)
 
-        # Check that including both capital and lowercase extensions raises an exception
+        # Check that including both capital and lowercase extensions raises an
+        # exception
         d = None
         try:
             d = tempfile.mkdtemp()
-            files = ("foo.SHP", "foo.SHX", "foo.PRJ", "foo.DBF", "foo.shp", "foo.shx", "foo.prj", "foo.dbf")
+            files = (
+                "foo.SHP",
+                "foo.SHX",
+                "foo.PRJ",
+                "foo.DBF",
+                "foo.shp",
+                "foo.shx",
+                "foo.prj",
+                "foo.dbf")
             for f in files:
                 path = os.path.join(d, f)
                 # open and immediately close to create empty file
@@ -539,14 +449,25 @@ class LayersTest(TestCase):
 
             # Only run the tests if this is a case sensitive OS
             if len(os.listdir(d)) == len(files):
-                self.assertRaises(GeoNodeException, lambda: get_files(os.path.join(d, "foo.SHP")))
-                self.assertRaises(GeoNodeException, lambda: get_files(os.path.join(d, "foo.shp")))
+                self.assertRaises(
+                    GeoNodeException,
+                    lambda: get_files(
+                        os.path.join(
+                            d,
+                            "foo.SHP")))
+                self.assertRaises(
+                    GeoNodeException,
+                    lambda: get_files(
+                        os.path.join(
+                            d,
+                            "foo.shp")))
 
         finally:
             if d is not None:
                 shutil.rmtree(d)
 
-        # Check that including both capital and lowercase PRJ (this is special-cased in the implementation)
+        # Check that including both capital and lowercase PRJ (this is
+        # special-cased in the implementation)
         d = None
         try:
             d = tempfile.mkdtemp()
@@ -558,17 +479,34 @@ class LayersTest(TestCase):
 
             # Only run the tests if this is a case sensitive OS
             if len(os.listdir(d)) == len(files):
-                self.assertRaises(GeoNodeException, lambda: get_files(os.path.join(d, "foo.SHP")))
-                self.assertRaises(GeoNodeException, lambda: get_files(os.path.join(d, "foo.shp")))
+                self.assertRaises(
+                    GeoNodeException,
+                    lambda: get_files(
+                        os.path.join(
+                            d,
+                            "foo.SHP")))
+                self.assertRaises(
+                    GeoNodeException,
+                    lambda: get_files(
+                        os.path.join(
+                            d,
+                            "foo.shp")))
         finally:
             if d is not None:
                 shutil.rmtree(d)
 
-        # Check that including both capital and lowercase SLD (this is special-cased in the implementation)
+        # Check that including both capital and lowercase SLD (this is
+        # special-cased in the implementation)
         d = None
         try:
             d = tempfile.mkdtemp()
-            files = ("foo.SHP", "foo.SHX", "foo.PRJ", "foo.DBF", "foo.SLD", "foo.sld")
+            files = (
+                "foo.SHP",
+                "foo.SHX",
+                "foo.PRJ",
+                "foo.DBF",
+                "foo.SLD",
+                "foo.sld")
             for f in files:
                 path = os.path.join(d, f)
                 # open and immediately close to create empty file
@@ -576,8 +514,18 @@ class LayersTest(TestCase):
 
             # Only run the tests if this is a case sensitive OS
             if len(os.listdir(d)) == len(files):
-                self.assertRaises(GeoNodeException, lambda: get_files(os.path.join(d, "foo.SHP")))
-                self.assertRaises(GeoNodeException, lambda: get_files(os.path.join(d, "foo.shp")))
+                self.assertRaises(
+                    GeoNodeException,
+                    lambda: get_files(
+                        os.path.join(
+                            d,
+                            "foo.SHP")))
+                self.assertRaises(
+                    GeoNodeException,
+                    lambda: get_files(
+                        os.path.join(
+                            d,
+                            "foo.shp")))
         finally:
             if d is not None:
                 shutil.rmtree(d)
@@ -611,17 +559,16 @@ class LayersTest(TestCase):
         self.assertRaises(GeoNodeException, get_valid_layer_name, 12, False)
         self.assertRaises(GeoNodeException, get_valid_layer_name, 12, True)
 
-    ## NOTE: we don't care about file content for many of these tests (the
-    ## forms under test validate based only on file name, and leave actual
-    ## content inspection to GeoServer) but Django's form validation will omit
-    ## any files with empty bodies.
-    ##
-    ## That is, this leads to mysterious test failures:
-    ##     SimpleUploadedFile('foo', '')
-    ##
-    ## And this should be used instead to avoid that:
-    ##     SimpleUploadedFile('foo', ' ')
-
+    # NOTE: we don't care about file content for many of these tests (the
+    # forms under test validate based only on file name, and leave actual
+    # content inspection to GeoServer) but Django's form validation will omit
+    # any files with empty bodies.
+    #
+    # That is, this leads to mysterious test failures:
+    #     SimpleUploadedFile('foo', '')
+    #
+    # And this should be used instead to avoid that:
+    #     SimpleUploadedFile('foo', ' ')
 
     def testJSONField(self):
         field = JSONField()
@@ -629,74 +576,75 @@ class LayersTest(TestCase):
         field.clean('{ "users": [] }')
 
         # text which is not JSON should fail
-        self.assertRaises(ValidationError, lambda: field.clean('<users></users>'))
+        self.assertRaises(
+            ValidationError,
+            lambda: field.clean('<users></users>'))
 
     def test_rating_layer_remove(self):
         """ Test layer rating is removed on layer remove
         """
-        #Get the layer to work with
+        # Get the layer to work with
         layer = Layer.objects.get(pk=3)
         layer.default_style = Style.objects.get(pk=layer.pk)
         layer.save()
         url = reverse('layer_remove', args=(layer.typename,))
         layer_id = layer.id
 
-        #Create the rating with the correct content type
+        # Create the rating with the correct content type
         ctype = ContentType.objects.get(model='layer')
-        OverallRating.objects.create(category=2,object_id=layer_id,content_type=ctype, rating=3)
+        OverallRating.objects.create(
+            category=2,
+            object_id=layer_id,
+            content_type=ctype,
+            rating=3)
 
-        c = Client()
+        self.client.login(username='admin', password='admin')
 
-        c.login(username='admin', password='admin')
+        # Remove the layer
+        self.client.post(url)
 
-        #Remove the layer
-        c.post(url)
-
-        #Check there are no ratings matching the remove layer
-        rating = OverallRating.objects.filter(category=2,object_id=layer_id)
-        self.assertEquals(rating.count(),0)
+        # Check there are no ratings matching the remove layer
+        rating = OverallRating.objects.filter(category=2, object_id=layer_id)
+        self.assertEquals(rating.count(), 0)
 
     def test_layer_remove(self):
         """Test layer remove functionality
         """
-        layer = Layer.objects.get(pk = 1)
+        layer = Layer.objects.get(pk=1)
         url = reverse('layer_remove', args=(layer.typename,))
         layer.default_style = Style.objects.get(pk=layer.pk)
         layer.save()
-        c = Client()
 
         # test unauthenticated
-        response = c.get(url)
+        response = self.client.get(url)
         self.assertEquals(response.status_code, 302)
 
-        #test a user without layer removal permission
-        c = Client()
-        c.login(username='norman', password='norman')
-        response = c.post(url)
+        # test a user without layer removal permission
+        self.client.login(username='norman', password='norman')
+        response = self.client.post(url)
         self.assertEquals(response.status_code, 302)
-        c.logout()
+        self.client.logout()
 
         # Now test with a valid user
-        c = Client()
-        c.login(username='admin', password='admin')
+        self.client.login(username='admin', password='admin')
 
-        #test a method other than POST and GET
-        response = c.put(url)
+        # test a method other than POST and GET
+        response = self.client.put(url)
         self.assertEquals(response.status_code, 403)
 
-        #test the page with a valid user with layer removal permission
-        response = c.get(url)
+        # test the page with a valid user with layer removal permission
+        response = self.client.get(url)
         self.assertEquals(response.status_code, 200)
 
-        #test the post method that actually removes the layer and redirects
-        response = c.post(url)
+        # test the post method that actually removes the layer and redirects
+        response = self.client.post(url)
         self.assertEquals(response.status_code, 302)
         self.assertEquals(response['Location'], 'http://testserver/layers/')
 
-        #test that the layer is actually removed
+        # test that the layer is actually removed
         self.assertEquals(Layer.objects.filter(pk=layer.pk).count(), 0)
-        
-        #test that all styles associated to the layer are removed
+
+        # test that all styles associated to the layer are removed
         self.assertEquals(Style.objects.count(), 0)
 
     def test_non_cascading(self):
@@ -716,70 +664,104 @@ class LayersTest(TestCase):
         self.assertEquals(layer1.default_style, layer2.default_style)
 
         # Now test with a valid user
-        c = Client()
-        c.login(username='admin', password='admin')
+        self.client.login(username='admin', password='admin')
 
-        #test the post method that actually removes the layer and redirects
-        response = c.post(url)
+        # test the post method that actually removes the layer and redirects
+        response = self.client.post(url)
         self.assertEquals(response.status_code, 302)
         self.assertEquals(response['Location'], 'http://testserver/layers/')
 
-        #test that the layer is actually removed
+        # test that the layer is actually removed
 
         self.assertEquals(Layer.objects.filter(pk=layer1.pk).count(), 0)
         self.assertEquals(Layer.objects.filter(pk=2).count(), 1)
 
-        #test that all styles associated to the layer are removed
+        # test that all styles associated to the layer are removed
         self.assertEquals(Style.objects.count(), 1)
 
     def test_category_counts(self):
         topics = TopicCategory.objects.all()
-        topics = topics.annotate(**{ 'layer_count': Count('resourcebase__layer__category')})
+        topics = topics.annotate(
+            **{'layer_count': Count('resourcebase__layer__category')})
         location = topics.get(identifier='location')
         # there are three layers with location category
-        self.assertEquals(location.layer_count,3)
+        self.assertEquals(location.layer_count, 3)
 
         # change the category of one layers_count
         layer = Layer.objects.filter(category=location)[0]
         elevation = topics.get(identifier='elevation')
         layer.category = elevation
         layer.save()
-        
-        #reload the categories since it's caching the old count
-        topics = topics.annotate(**{ 'layer_count': Count('resourcebase__layer__category')})
+
+        # reload the categories since it's caching the old count
+        topics = topics.annotate(
+            **{'layer_count': Count('resourcebase__layer__category')})
         location = topics.get(identifier='location')
         elevation = topics.get(identifier='elevation')
-        self.assertEquals(location.layer_count,2)
-        self.assertEquals(elevation.layer_count,4)
+        self.assertEquals(location.layer_count, 2)
+        self.assertEquals(elevation.layer_count, 4)
 
         # delete a layer and check the count update
         # use the first since it's the only one which has styles
-        layer =  Layer.objects.get(pk=1)
+        layer = Layer.objects.get(pk=1)
         elevation = topics.get(identifier='elevation')
-        self.assertEquals(elevation.layer_count,4)
+        self.assertEquals(elevation.layer_count, 4)
         layer.delete()
-        topics = topics.annotate(**{ 'layer_count': Count('resourcebase__layer__category')})
+        topics = topics.annotate(
+            **{'layer_count': Count('resourcebase__layer__category')})
         elevation = topics.get(identifier='elevation')
-        self.assertEquals(elevation.layer_count,3)
+        self.assertEquals(elevation.layer_count, 3)
 
-    def test_not_superuser_permissions(self):
-        #grab bobby
-        bob = User.objects.get(username='bobby')
 
-        #grab a layer
-        layer = Layer.objects.all()[0]
+class UnpublishedObjectTests(TestCase):
 
-        #set layer permissions to registered read/write
-        layer.set_gen_level(AUTHENTICATED_USERS,'layer_readwrite')
+    """Test the is_published base attribute"""
 
-        #verify bobby has view/change permissions on it
-        self.assertTrue(bob.has_perm('layers.view_layer',layer))
-        self.assertTrue(bob.has_perm('layers.change_layer',layer))
+    fixtures = ['bobby']
 
-        #verify that bobby can access the layer data page
-        c = Client()
-        self.assertTrue(c.login(username='bobby',password='bob'))
+    def setUp(self):
+        super(UnpublishedObjectTests, self).setUp()
 
-        response = c.get(reverse('layer_detail', args=(layer.typename,)))
-        self.assertEquals(response.status_code, 200)
+        self.list_url = reverse(
+            'api_dispatch_list',
+            kwargs={
+                'api_name': 'api',
+                'resource_name': 'layers'})
+        create_models(type='layer')
+        all_public()
 
+    def test_unpublished_layer(self):
+        """Test unpublished layer behaviour"""
+
+        user = get_user_model().objects.get(username='bobby')
+        self.client.login(username='bobby', password='bob')
+
+        # default (RESOURCE_PUBLISHING=False)
+        # access to layer detail page gives 200 if layer is published or
+        # unpublished
+        response = self.client.get(reverse('layer_detail', args=('geonode:CA',)))
+        self.failUnlessEqual(response.status_code, 200)
+        layer = Layer.objects.filter(title='CA')[0]
+        layer.is_published = False
+        layer.save()
+        response = self.client.get(reverse('layer_detail', args=('geonode:CA',)))
+        self.failUnlessEqual(response.status_code, 200)
+
+        # with resource publishing
+        with self.settings(RESOURCE_PUBLISHING=True):
+            # 404 if layer is unpublished
+            response = self.client.get(reverse('layer_detail', args=('geonode:CA',)))
+            self.failUnlessEqual(response.status_code, 404)
+            # 200 if layer is unpublished but user has permission
+            assign_perm('publish_resourcebase', user, layer.get_self_resource())
+            response = self.client.get(reverse('layer_detail', args=('geonode:CA',)))
+            self.failUnlessEqual(response.status_code, 200)
+            # 200 if layer is published
+            layer.is_published = True
+            layer.save()
+            remove_perm('publish_resourcebase', user, layer.get_self_resource())
+            response = self.client.get(reverse('layer_detail', args=('geonode:CA',)))
+            self.failUnlessEqual(response.status_code, 200)
+
+        layer.is_published = True
+        layer.save()
