@@ -19,39 +19,36 @@
 #########################################################################
 
 import logging
-import math
-import errno
 import uuid
-import httplib2
-from urlparse import urlparse
-import urllib
 
 from django.conf import settings
 from django.db import models
 from django.db.models import signals
 from django.utils import simplejson as json
-from django.contrib.auth.models import User, AnonymousUser
 from django.contrib.contenttypes.models import ContentType
 from django.utils.translation import ugettext_lazy as _
 from django.core.exceptions import ObjectDoesNotExist
 from django.core.urlresolvers import reverse
 from django.template.defaultfilters import slugify
+from django.core.cache import cache
 
 from geonode.layers.models import Layer
-from geonode.base.models import ResourceBase
+from geonode.base.models import ResourceBase, resourcebase_post_save
 from geonode.maps.signals import map_changed_signal
-from geonode.security.enumerations import AUTHENTICATED_USERS, ANONYMOUS_USERS
 from geonode.utils import GXPMapBase
 from geonode.utils import GXPLayerBase
 from geonode.utils import layer_from_viewer_config
 from geonode.utils import default_map_config
-from geonode.utils import forward_mercator
+from geonode.utils import num_encode
+from geonode.security.models import remove_object_permissions
 
 from agon_ratings.models import OverallRating
 
 logger = logging.getLogger("geonode.maps.models")
 
+
 class Map(ResourceBase, GXPMapBase):
+
     """
     A Map aggregates several layers together and annotates them with a viewport
     configuration.
@@ -62,7 +59,7 @@ class Map(ResourceBase, GXPMapBase):
     # The zoom level to use when initially loading this map.  Zoom levels start
     # at 0 (most zoomed out) and each increment doubles the resolution.
 
-    projection = models.CharField(_('projection'),max_length=32)
+    projection = models.CharField(_('projection'), max_length=32)
     # The projection used for this map.  This is stored as a string with the
     # projection's SRID.
 
@@ -77,11 +74,19 @@ class Map(ResourceBase, GXPMapBase):
     last_modified = models.DateTimeField(auto_now_add=True)
     # The last time the map was modified.
 
-    popular_count = models.IntegerField(default=0)
-    share_count = models.IntegerField(default=0)
+    urlsuffix = models.CharField(_('Site URL'), max_length=255, blank=True)
+    # Alphanumeric alternative to referencing maps by id, appended to end of
+    # URL instead of id, ie http://domain/maps/someview
+
+    featuredurl = models.CharField(
+        _('Featured Map URL'),
+        max_length=255,
+        blank=True)
+    # Full URL for featured map view, ie http://domain/someview
 
     def __unicode__(self):
-        return '%s by %s' % (self.title, (self.owner.username if self.owner else "<Anonymous>"))
+        return '%s by %s' % (
+            self.title, (self.owner.username if self.owner else "<Anonymous>"))
 
     @property
     def center(self):
@@ -100,7 +105,7 @@ class Map(ResourceBase, GXPMapBase):
     def local_layers(self):
         layer_names = MapLayer.objects.filter(map__id=self.id).values('name')
         return Layer.objects.filter(typename__in=layer_names) | \
-               Layer.objects.filter(name__in=layer_names)
+            Layer.objects.filter(name__in=layer_names)
 
     def json(self, layer_filter):
         """
@@ -111,7 +116,7 @@ class Map(ResourceBase, GXPMapBase):
         layers = []
         for map_layer in map_layers:
             if map_layer.local:
-                layer =  Layer.objects.get(typename=map_layer.name)
+                layer = Layer.objects.get(typename=map_layer.name)
                 layers.append(layer)
             else:
                 pass
@@ -122,9 +127,16 @@ class Map(ResourceBase, GXPMapBase):
         # the readme text will appear in a README file in the zip
         readme = (
             "Title: %s\n" +
-            "Author: %s\n"
+            "Author: %s\n" +
             "Abstract: %s\n"
-        ) % (self.title, "The GeoNode Team", self.abstract)
+        ) % (self.title, self.poc, self.abstract)
+        if self.license:
+            readme += "License: %s" % self.license
+            if self.license.url:
+                readme += " (%s)" % self.license.url
+            readme += "\n"
+        if self.constraints_other:
+            readme += "Additional constraints: %s\n" % self.constraints_other
 
         def layer_json(lyr):
             return {
@@ -136,8 +148,8 @@ class Map(ResourceBase, GXPMapBase):
 
         map_config = {
             # the title must be provided and is used for the zip file name
-            "map" : { "readme": readme, "title": self.title },
-            "layers" : [layer_json(lyr) for lyr in layers]
+            "map": {"readme": readme, "title": self.title},
+            "layers": [layer_json(lyr) for lyr in layers]
         }
 
         return json.dumps(map_config)
@@ -155,10 +167,10 @@ class Map(ResourceBase, GXPMapBase):
         self.title = conf['about']['title']
         self.abstract = conf['about']['abstract']
 
-        self.zoom = conf['map']['zoom']
-
-        self.center_x = conf['map']['center'][0]
-        self.center_y = conf['map']['center'][1]
+        self.set_bounds_from_center_and_zoom(
+            conf['map']['center'][0],
+            conf['map']['center'][1],
+            conf['map']['zoom'])
 
         self.projection = conf['map']['projection']
 
@@ -180,14 +192,12 @@ class Map(ResourceBase, GXPMapBase):
             self.layer_set.add(
                 layer_from_viewer_config(
                     MapLayer, layer, source_for(layer), ordering
-            ))
-
-        self.set_bounds_from_layers(self.local_layers)
+                ))
 
         self.save()
 
         if layer_names != set([l.typename for l in self.local_layers]):
-            map_changed_signal.send_robust(sender=self,what_changed='layers')
+            map_changed_signal.send_robust(sender=self, what_changed='layers')
 
     def keyword_list(self):
         keywords_qs = self.keywords.all()
@@ -199,41 +209,9 @@ class Map(ResourceBase, GXPMapBase):
     def get_absolute_url(self):
         return reverse('geonode.maps.views.map_detail', None, [str(self.id)])
 
-
-    class Meta:
-        # custom permissions,
-        # change and delete are standard in django
-        permissions = (('view_map', 'Can view'),
-                       ('change_map_permissions', "Can change permissions"), )
-
-    # Permission Level Constants
-    # LEVEL_NONE inherited
-    LEVEL_READ  = 'map_readonly'
-    LEVEL_WRITE = 'map_readwrite'
-    LEVEL_ADMIN = 'map_admin'
-
-    def set_default_permissions(self):
-        self.set_gen_level(ANONYMOUS_USERS, self.LEVEL_READ)
-        self.set_gen_level(AUTHENTICATED_USERS, self.LEVEL_READ)
-
-        # remove specific user permissions
-        current_perms = self.get_all_level_info()
-        for username in current_perms['users'].keys():
-            user = User.objects.get(username=username)
-            self.set_user_level(user, self.LEVEL_NONE)
-
-        # assign owner admin privs
-        if self.owner:
-            self.set_user_level(self.owner, self.LEVEL_ADMIN)
-
-    def get_extent(self):
-        """Generate minx/miny/maxx/maxy of map extent"""
-
-        return self.bbox
-
-    def set_bounds_from_layers(self, layers):
+    def get_bbox_from_layers(self, layers):
         """
-        Calculate the bounds from a given list of Layer objects
+        Calculate the bbox from a given list of Layer objects
         """
         bbox = None
         for layer in layers:
@@ -245,27 +223,6 @@ class Map(ResourceBase, GXPMapBase):
                 bbox[1] = max(bbox[1], layer_bbox[1])
                 bbox[2] = min(bbox[2], layer_bbox[2])
                 bbox[3] = max(bbox[3], layer_bbox[3])
-        else:
-            return None
-        
-        self.bbox_x0 = bbox[0]
-        self.bbox_x1 = bbox[1]
-        self.bbox_y0 = bbox[2]
-        self.bbox_y1 = bbox[3]
-
-        minx, miny, maxx, maxy = [float(c) for c in bbox]
-        x = (minx + maxx) / 2
-        y = (miny + maxy) / 2
-        (center_x, center_y) = forward_mercator((x,y))
-
-        width_zoom = math.log(360 / (maxx - minx), 2)
-        height_zoom = math.log(360 / (maxy - miny), 2)
-
-        zoom = math.ceil(min(width_zoom, height_zoom))
-
-        self.zoom = zoom
-        self.center_x = center_x
-        self.center_y = center_y
 
         return bbox
 
@@ -273,11 +230,10 @@ class Map(ResourceBase, GXPMapBase):
         self.owner = user
         self.title = title
         self.abstract = abstract
-        self.projection="EPSG:900913"
+        self.projection = "EPSG:900913"
         self.zoom = 0
         self.center_x = 0
         self.center_y = 0
-        map_layers = []
         bbox = None
         index = 0
 
@@ -292,42 +248,59 @@ class Map(ResourceBase, GXPMapBase):
                 try:
                     layer = Layer.objects.get(typename=layer)
                 except ObjectDoesNotExist:
-                    raise GeoNodeError('Could not find layer with name %s' % layer)
+                    raise Exception(
+                        'Could not find layer with name %s' %
+                        layer)
 
-            if not user.has_perm('maps.view_layer', obj=layer):
+            if not user.has_perm(
+                    'base.view_resourcebase',
+                    obj=layer.resourcebase_ptr):
                 # invisible layer, skip inclusion or raise Exception?
-                raise GeoNodeError('User %s tried to create a map with layer %s without having premissions' % (user, layer))
+                raise Exception(
+                    'User %s tried to create a map with layer %s without having premissions' %
+                    (user, layer))
             MapLayer.objects.create(
-                map = self,
-                name = layer.typename,
-                ows_url = layer.get_ows_url(),
-                stack_order = index,
-                visibility = True
+                map=self,
+                name=layer.typename,
+                ows_url=layer.get_ows_url(),
+                stack_order=index,
+                visibility=True
             )
 
             index += 1
 
         # Set bounding box based on all layers extents.
-        bbox = self.set_bounds_from_layers(self.local_layers)
+        bbox = self.get_bbox_from_layers(self.local_layers)
+
+        self.set_bounds_from_bbox(bbox)
 
         self.set_missing_info()
 
         # Save again to persist the zoom and bbox changes and
         # to generate the thumbnail.
         self.save()
-        
 
     @property
     def class_name(self):
         return self.__class__.__name__
 
+    @property
+    def snapshots(self):
+        snapshots = MapSnapshot.objects.exclude(
+            user=None).filter(
+            map__id=self.map.id)
+        return [snapshot for snapshot in snapshots]
+
     @property
     def is_public(self):
         """
         Returns True if anonymous (public) user can view map.
         """
-        user = AnonymousUser()
-        return user.has_perm('maps.view_map', obj=self)
+        from guardian.shortcuts import get_anonymous_user
+        user = get_anonymous_user()
+        return user.has_perm(
+            'base.view_resourcebase',
+            obj=self.resourcebase_ptr)
 
     @property
     def layer_group(self):
@@ -349,15 +322,16 @@ class Map(ResourceBase, GXPMapBase):
             from geonode.geoserver.helpers import gs_catalog
             from geoserver.layergroup import UnsavedLayerGroup as GsUnsavedLayerGroup
         else:
-            raise Exception('Cannot publish layer group if geonode.geoserver is not in INSTALLED_APPS')
+            raise Exception(
+                'Cannot publish layer group if geonode.geoserver is not in INSTALLED_APPS')
 
-        # temporary permission workaround: 
+        # temporary permission workaround:
         # only allow public maps to be published
         if not self.is_public:
             return 'Only public maps can be saved as layer group.'
 
         map_layers = MapLayer.objects.filter(map=self.id)
-        
+
         # Local Group Layer layers and corresponding styles
         layers = []
         lg_styles = []
@@ -369,21 +343,30 @@ class Map(ResourceBase, GXPMapBase):
                 lg_styles.append(style)
         lg_layers = [l.name for l in layers]
 
-        # Group layer bounds and name             
-        lg_bounds = [str(coord) for coord in self.bbox] 
+        # Group layer bounds and name
+        lg_bounds = [str(coord) for coord in self.bbox]
         lg_name = '%s_%d' % (slugify(self.title), self.id)
 
         # Update existing or add new group layer
         lg = self.layer_group
         if lg is None:
-            lg = GsUnsavedLayerGroup(gs_catalog, lg_name, lg_layers, lg_styles, lg_bounds)
+            lg = GsUnsavedLayerGroup(
+                gs_catalog,
+                lg_name,
+                lg_layers,
+                lg_styles,
+                lg_bounds)
         else:
             lg.layers, lg.styles, lg.bounds = lg_layers, lg_styles, lg_bounds
         gs_catalog.save(lg)
         return lg_name
 
+    class Meta(ResourceBase.Meta):
+        pass
+
 
 class MapLayer(models.Model, GXPLayerBase):
+
     """
     The MapLayer model represents a layer included in a map.  This doesn't just
     identify the dataset, but also extra options such as which style to load
@@ -397,7 +380,11 @@ class MapLayer(models.Model, GXPLayerBase):
     # The z-index of this layer in the map; layers with a higher stack_order will
     # be drawn on top of others.
 
-    format = models.CharField(_('format'), null=True, max_length=200, blank=True)
+    format = models.CharField(
+        _('format'),
+        null=True,
+        max_length=200,
+        blank=True)
     # The mimetype of the image format to use for tiles (image/png, image/jpeg,
     # image/gif...)
 
@@ -411,17 +398,22 @@ class MapLayer(models.Model, GXPLayerBase):
     opacity = models.FloatField(_('opacity'), default=1.0)
     # The opacity with which to render this layer, on a scale from 0 to 1.
 
-    styles = models.CharField(_('styles'), null=True,max_length=200, blank=True)
+    styles = models.CharField(
+        _('styles'),
+        null=True,
+        max_length=200,
+        blank=True)
     # The name of the style to use for this layer (only useful for WMS layers.)
 
-    transparent = models.BooleanField(_('transparent'))
-    # A boolean value, true if we should request tiles with a transparent background.
+    transparent = models.BooleanField(_('transparent'), default=False)
+    # A boolean value, true if we should request tiles with a transparent
+    # background.
 
     fixed = models.BooleanField(_('fixed'), default=False)
     # A boolean value, true if we should prevent the user from dragging and
     # dropping this layer in the layer chooser.
 
-    group = models.CharField(_('group'), null=True,max_length=200, blank=True)
+    group = models.CharField(_('group'), null=True, max_length=200, blank=True)
     # A group label to apply to this layer.  This affects the hierarchy displayed
     # in the map viewer's layer tree.
 
@@ -448,22 +440,49 @@ class MapLayer(models.Model, GXPLayerBase):
     local = models.BooleanField(default=False)
     # True if this layer is served by the local geoserver
 
-    def layer_config(self):
-        cfg = GXPLayerBase.layer_config(self)
+    def layer_config(self, user=None):
+        # Try to use existing user-specific cache of layer config
+        if self.id:
+            cfg = cache.get("layer_config" +
+                            str(self.id) +
+                            "_" +
+                            str(0 if user is None else user.id))
+            if cfg is not None:
+                return cfg
 
+        cfg = GXPLayerBase.layer_config(self, user=user)
         # if this is a local layer, get the attribute configuration that
         # determines display order & attribute labels
-        if self.local:
-            if Layer.objects.filter(typename=self.name).exists():
-                layer = Layer.objects.get(typename=self.name)
+        if Layer.objects.filter(typename=self.name).exists():
+            try:
+                if self.local:
+                    layer = Layer.objects.get(typename=self.name)
+                else:
+                    layer = Layer.objects.get(
+                        typename=self.name,
+                        service__base_url=self.ows_url)
                 attribute_cfg = layer.attribute_config()
                 if "getFeatureInfo" in attribute_cfg:
                     cfg["getFeatureInfo"] = attribute_cfg["getFeatureInfo"]
-            else:
-                # shows maplayer with pink tiles, 
+                if not user.has_perm(
+                        'base.view_resourcebase',
+                        obj=layer.resourcebase_ptr):
+                    cfg['disabled'] = True
+                    cfg['visibility'] = False
+            except:
+                # shows maplayer with pink tiles,
                 # and signals that there is problem
                 # TODO: clear orphaned MapLayers
                 layer = None
+
+        if self.id:
+            # Create temporary cache of maplayer config, should not last too long in case
+            # local layer permissions or configuration values change (default
+            # is 5 minutes)
+            cache.set("layer_config" +
+                      str(self.id) +
+                      "_" +
+                      str(0 if user is None else user.id), cfg)
         return cfg
 
     @property
@@ -478,7 +497,8 @@ class MapLayer(models.Model, GXPLayerBase):
     def local_link(self):
         if self.local:
             layer = Layer.objects.get(typename=self.name)
-            link = "<a href=\"%s\">%s</a>" % (layer.get_absolute_url(),layer.title)
+            link = "<a href=\"%s\">%s</a>" % (
+                layer.get_absolute_url(), layer.title)
         else:
             link = "<span>%s</span> " % self.name
         return link
@@ -492,8 +512,40 @@ class MapLayer(models.Model, GXPLayerBase):
 
 def pre_delete_map(instance, sender, **kwrargs):
     ct = ContentType.objects.get_for_model(instance)
-    OverallRating.objects.filter(content_type = ct, object_id = instance.id).delete()
+    OverallRating.objects.filter(
+        content_type=ct,
+        object_id=instance.id).delete()
+    remove_object_permissions(instance.get_self_resource())
 
 
+class MapSnapshot(models.Model):
+    map = models.ForeignKey(Map, related_name="snapshot_set")
+    """
+    The ID of the map this snapshot was generated from.
+    """
+
+    config = models.TextField(_('JSON Configuration'))
+    """
+    Map configuration in JSON format
+    """
+
+    created_dttm = models.DateTimeField(auto_now_add=True)
+    """
+    The date/time the snapshot was created.
+    """
+
+    user = models.ForeignKey(settings.AUTH_USER_MODEL, blank=True, null=True)
+    """
+    The user who created the snapshot.
+    """
+
+    def json(self):
+        return {
+            "map": self.map.id,
+            "created": self.created_dttm.isoformat(),
+            "user": self.user.username if self.user else None,
+            "url": num_encode(self.id)
+        }
 
 signals.pre_delete.connect(pre_delete_map, sender=Map)
+signals.post_save.connect(resourcebase_post_save, sender=Map)
